package org.sc3d.apt.sss.v3;

/** A tool for checking that SSS files obey a grammar. */
public class Validator extends Parser {
  /** Don't want any instances of this class. */
  private Validator(Grammar g) { super(g); }
  
  /* Override things in Parser. */
  
  /** Descends into all brackets to make sure everything gets parsed. Always returns 'null'. */
  public Object postProcess(Tree raw) {
    if (raw.isTerminal) {
      final Tree.Terminal rawT = (Tree.Terminal)raw;
      final int type = rawT.t.type;
      if (
        type==Token.TYPE_ROUND ||
        type==Token.TYPE_SQUARE ||
        type==Token.TYPE_BRACE
      ) {
        final Tree contents = rawT.parse();
        if (contents!=null) this.postProcess(contents);
      }
    } else {
      final Tree.NonTerminal rawNT = (Tree.NonTerminal)raw;
      for (int i=0; i<rawNT.length; i++) {
        final Tree.Production rawP = rawNT.get(i);
        for (int j=0; j<rawP.numParts; j++) {
          this.postProcess(rawP.get(j));
        }
      }
    }
    return null;
  }
  
  /* Test code. */
  
  /** First argument is the filename of the grammar specification. Subsequent filenames are the filenames of files obeying the grammar. */
  public static void main(String[] args) throws java.io.IOException {
    if (args.length<1) throw new IllegalArgumentException(
      "Syntax: java org.sc3d.apt.sss.v3.Validator "+
      "<filename of grammar> <file1> <file2> ..."
    );
    final Grammar g = GrammarParser.fromFile(args[0]);
    System.out.println("Grammar is okay.\n");
    final Validator me = new Validator(g);
    if (args.length==1) System.out.println(me.ndfa);
    for (int i=1; i<args.length; i++) {
      System.out.print("File '"+args[i]+"' has ");
      final Sentence s = Sentence.readFile(args[i]);
      me.parse(s);
      System.out.println(s.countErrors()+" errors.\n");
      if (s.countErrors()!=0) s.printErrorReport(System.out, 1000000);
    }
  }
}
